==> nono-0.4.4.tar.gz: nono-0.4.4/vm/vm_luna.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// VM (LUNA)
//

#include "vm_luna.h"
#include "bus.h"
#include "bt454.h"
#include "busio.h"
#include "cmmu.h"
#include "config.h"
#include "crtc2.h"
#include "dipsw.h"
#include "ethernet.h"
#include "fdc.h"
#include "fuserom.h"
#include "interrupt.h"
#include "lance.h"
#include "lcd.h"
#include "lunafb.h"
#include "lunakbd.h"
#include "mainapp.h"
#include "mainram.h"
#include "mk48t02.h"
#include "mpu680x0.h"
#include "mpu88xx0.h"
#include "nmi.h"
#include "partialram.h"
#include "pio.h"
#include "power.h"
#include "prom.h"
#include "renderer.h"
#include "romemu_luna1.h"
#include "romemu_luna88k.h"
#include "scheduler.h"
#include "sio.h"
#include "spc.h"
#include "subram.h"
#include "sysclk.h"
#include "sysctlr.h"
#include "tas.h"

//
// LUNA 共通部
//

// デバイスマップ用の識別子
enum {
	NONE,
	BUSERR,
	PROM,
	MK48,
	PIO0,
	PIO1,
	SIO,
	TAS,
	SYSCLK,
	SUBRAM,
	MROM,
	LUNAFB,
	BT454,
	CRTC2,
	SCSI,
	LANCE,
	FUSE,
	SYSCTL,
	CMMU,
};

// コンストラクタ
VM_LUNA::VM_LUNA()
{
	// RTC epoch は NetBSD/luna68k, OpenBSD/luna88k をデフォルトにしておく。
	gConfig->SetDefault("rtc-epoch-year", 1970);

	// LUNA 共通だしここで作成しておく
	NEWDV(Dipsw, new LunaDipswDevice());
	NEWDV(NopIO, new NopIODevice());
}

// デストラクタ
VM_LUNA::~VM_LUNA()
{
}

// デバイスマップから devtable を作成 (ここは共通にできる)
void
VM_LUNA::MakeDevtable(const uint8 *maptable)
{
	for (int i = 0; i < gMainbus->devtable.size(); i++) {
		IODevice *d;
		switch (maptable[i]) {
		 case NONE:		d = gNopIO;		break;
		 case BUSERR:	d = gBusErr;	break;
		 case PROM:		d = gPROM;		break;
		 case MK48:		d = gRTC;		break;
		 case PIO0:		d = gPIO0;		break;
		 case PIO1:		d = gPIO1;		break;
		 case SIO:		d = gSIO;		break;
		 case TAS:		d = gTAS;		break;
		 case SYSCLK:	d = gSysClk;	break;
		 case SUBRAM:	d = gSubRAM;	break;
		 case MROM:		d = gNopIO;		break;
		 case LUNAFB:	d = gPlaneVRAM;	break;
		 case BT454:	d = gBT454;		break;
		 case CRTC2:	d = gCRTC2;		break;
		 case SCSI:		d = gSPC;		break;
		 case LANCE:	d = gEthernet;	break;
		 case FUSE:		d = gFuseROM;	break;
		 case SYSCTL:	d = gInterrupt;	break;
		 case CMMU:		d = gCMMU;		break;
		 default:
			__unreachable();
		}
		gMainbus->SetDevice(i, d);
	}
}

// 初期化
bool
VM_LUNA::Init()
{
	// 親クラス
	if (inherited::Init() == false) {
		return false;
	}

	// RAM 容量が確定したので、ここで devtable に RAM を配置。
	int ram_size_MB = gMainRAM->GetSizeMB();
	int i = 0;
	while (ram_size_MB >= 16) {
		// 16MB あれば RAM が直接担当
		gMainbus->SetDevice(i++, gMainRAM);
		ram_size_MB -= 16;
	}
	// 16MB 未満なら PartialRAM が担当
	if (ram_size_MB > 0) {
		gMainbus->SetDevice(i, gPartialRAM);
	}

	// ブートページ切り替え用に覚えておく
	ram0 = gMainbus->devtable[0];

	return true;
}

// ブートページを切り替える。
void
VM_LUNA::SwitchBootPage(Device *parent, bool isrom)
{
	// 共通処理
	inherited::SwitchBootPage(parent, isrom);

	// ROM は 128KB しかないので 16MB まるごと切り替わったりはしないとは
	// 思うけど、具体的にどの範囲が切り替わるのかを知る方法がないのと
	// 大して副作用もないので、先頭の 1ブロックをまるごと切り替えておく。
	if (isrom) {
		gMainbus->SetDevice(0x00, gPROM0);
		gMainbus->SetDevice(0x41, gPROM0);
	} else {
		gMainbus->SetDevice(0x00, ram0);
		gMainbus->SetDevice(0x41, gPROM);
	}
}


//
// LUNA-I
//

// LUNA-I のデバイス割り当て表 (上位8ビット)
//
// 仕様	実際
// 00	00		メインメモリ
// --	01..03	二重バスフォールトするようだ
// --	04..0f	何かよく分からないものが読める
// --	10..3f	BUSERR
// 41	40..43	PROM (B、N=128KBでミラー)
// 45	44..47	mk48t02 (B、N=2048でミラー)
// 49	48..4b	pio0 (B、N=4でミラー)
// 4d	4c..4f	pio1 (B、N=4でミラー)
// 51	50..53	sio  (BB、N=4でミラー)
// --	54..5f	0xff が読めるようだ
// 61	60..61	TAS (B、N=1でミラー)
// 63	62..63	システムクロック (B、N=1でミラー)
// --           以降TASとSysClkで2枠ずつ。また無効ビットの読み出しは
// --	        必ずしも Hi とは限らず不安定のようだ。
// 71	70..7f	3ポートRAM (B、N=128KBでミラー)
// 81	80..	BUSERR (オンボード拡張スロットA)
// 83	  ..8f	BUSERR (オンボード拡張スロットB)
// 91	90..9f	0xff が読めるようだ (PC-9000 拡張スロット)
//
// a1			マスクROM
//              a0,a2,a4,... BUSERR
//              a1,a3,a5,... マスクROM(なければ0xffが読める)
// b1			ビットマップ
//              b0,b2,b4,... BUSERR
//              b1,b3,b5,... ビットマップ (内訳は lunafb.cpp 参照)
// c1			パレット
//              c0,c2,c4,... BUSERR
//              c1,c3,c5,... パレット (B、N=4でミラー)
// d1			CRTC-II
//              d0,d2,d4,... BUSERR
//              d1,d3,d5,... CRTC-II (B、N=2でミラー)
//
// e1	e0..e3	SCSI (BBBB、N=16 でミラー)
// --	e4..ef	0xff が読めるようだが不安定。
// f1	f0..f3	LANCE (WW、N=2 でミラー)
// --	f4..ff	0xff が読めるようだ

/*static*/ const uint8
VM_LUNA1::maptable[256] = {
	// (00,)01,02,03 は実際にはたぶんパリティチェックが入っていて(?)
	// 非実装領域にアクセスすると二重バスフォールトになるがそれは未対応。
	// RAM は Init() で動的に配置する。
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 00
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 08
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 10
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 18
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 20
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 28
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 30
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 38

	PROM,	PROM,	PROM,	PROM,	MK48,	MK48,	MK48,	MK48,	// 40
	PIO0,	PIO0,	PIO0,	PIO0,	PIO1,	PIO1,	PIO1,	PIO1,	// 48
	SIO,	SIO,	SIO,	SIO,	NONE,	NONE,	NONE,	NONE,	// 50
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 58
	TAS,	TAS,	SYSCLK,	SYSCLK,	TAS,	TAS,	SYSCLK,	SYSCLK,	// 60
	TAS,	TAS,	SYSCLK,	SYSCLK,	TAS,	TAS,	SYSCLK,	SYSCLK,	// 68
	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	// 70
	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	// 78

	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 80
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 88
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 90
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 98
	BUSERR,	MROM,	BUSERR,	MROM,	BUSERR,	MROM,	BUSERR,	MROM,	// a0
	BUSERR,	MROM,	BUSERR,	MROM,	BUSERR,	MROM,	BUSERR,	MROM,	// a8
	BUSERR,	LUNAFB,	BUSERR,	LUNAFB,	BUSERR,	LUNAFB,	BUSERR,	LUNAFB,	// b0
	BUSERR,	LUNAFB,	BUSERR,	LUNAFB,	BUSERR,	LUNAFB,	BUSERR,	LUNAFB,	// b8

	BUSERR,	BT454,	BUSERR,	BT454,	BUSERR,	BT454,	BUSERR,	BT454,	// c0
	BUSERR,	BT454,	BUSERR,	BT454,	BUSERR,	BT454,	BUSERR,	BT454,	// c8
	BUSERR,	CRTC2,	BUSERR,	CRTC2,	BUSERR,	CRTC2,	BUSERR,	CRTC2,	// d0
	BUSERR,	CRTC2,	BUSERR,	CRTC2,	BUSERR,	CRTC2,	BUSERR,	CRTC2,	// d8
	SCSI,	SCSI,	SCSI,	SCSI,	NONE,	NONE,	NONE,	NONE,	// e0
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// e8
	LANCE,	LANCE,	LANCE,	LANCE,	NONE,	NONE,	NONE,	NONE,	// f0
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// f8
};

// コンストラクタ
VM_LUNA1::VM_LUNA1()
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 20);
	// RAM 初期値は MAX にしておく
	gConfig->SetDefault("ram-size", 16);
	// SPC の入力クロック (おそらく 6.144MHz = 163ns が入ってる)
	gConfig->Add("!spc-tCLF", 163);
	// LUNA-I のクロック割り込みは 60Hz
	gConfig->Add("!sysclk-period", 16.666667_msec);

	// デバイス (順序に注意)
	// BusErrDevice, NopIODevice は親コンストラクタで作成済み。

	NEWDV(MPU, new MPU680x0Device());
	NEWDV(Interrupt, new LunaInterrupt());
	if (gMainApp.exec_file || gConfig->Find("prom-image").AsString().empty()) {
		NEWDV(PROM, new Luna1PROMEmuDevice());
	} else {
		NEWDV(PROM, new PROMDevice());				// required by RAM,BT454
	}
	NEWDV(PROM0, new PROM0Device());
	NEWDV(MainRAM, new MainRAMDevice());			// required by PartialRAM
	NEWDV(PartialRAM, new PartialRAMDevice());
	NEWDV(RTC, new BusIO_B<MK48T02Device>());		// required by Sysclk
	NEWDV(PIO0, new BusIO_B<PIO0Device>());
	NEWDV(PIO1, new BusIO_B<PIO1Device>());
	NEWDV(SIO, new BusIO_BB<SIODevice>());
	NEWDV(SysClk, new BusIO_B<SysClkDevice>());
	NEWDV(TAS, new BusIO_B<TASDevice>());
	NEWDV(SubRAM, new SubRAMDevice(128));
	NEWDV(BT454, new BT454Device());				// required by Lunafb
	NEWDV(PlaneVRAM, new LunafbDevice());
	NEWDV(CRTC2, new BusIO_B<CRTC2Device>());
	NEWDV(SPC, new BusIO_BBBB<SPCDevice>());
	NEWDV(Ethernet, new BusIO_W<LanceDevice>());

	NEWDV(NMI, new NMIDevice());
	NEWDV(LCD, new LCDDevice());
	NEWDV(Keyboard, new LunaKeyboard());
	NEWDV(Renderer, new LunaRenderer());

	// デバイスマップから devtable を作成
	MakeDevtable(maptable);
}

// デストラクタ
VM_LUNA1::~VM_LUNA1()
{
}


//
// LUNA-88K
//

// 00..03		RAM (64MB 実機の場合)
// 04..3f		アドレスの残像? (64MB 実機の場合)
// 40,41		PROM (B、N=256KB でミラー)
// 42,43		FUSEROM (BFFF、N=256 でミラー)
// 44,45		NVRAM (BFFF、N=2048 でミラー)
// 46,47		アドレスの残像?
// 48,49		PIO0 (BFFF、N=4 でミラー)
// 4a,4b		アドレスの残像?
// 4c,4d		PIO1 (BFFF、N=4 でミラー)
// 4e,4f		アドレスの残像?
// 50,51		SIO (BFFF、N=4 でミラー)
// 52..5f		アドレスの残像?
// 60,61		TAS? (BFFF、N=1 ?)
// 62,63		SysClk (BFFF、N=1 ?)
// 64,65,66,67	SysCtlr? (BFFF、N=4 ?)
// 68,69,6a,6b	SysCtlr? (L、N=4 ?)
// 6c,6d		CPU リセット
// 6e,6f		68-6b と同じパターンの気がする
// 70-7f		SubRAM
// 80-9f		アドレスの残像?
// a0-af		偶数(a0,a2,..)は残像、奇数(a1,a3,..)は ff
// b0-bf		偶数(b0,b2,..)は残像、奇数(b1,b3,..)は LUNAFB
// c0-cf		偶数(c0,c2,..)は残像、奇数(c1,c3,..)は BT454(BFFF、N=4)
// d0-df		偶数(d0,d2,..)は残像、奇数(d1,d3,..)はたぶん CRTC2
//				(CRTC2 は今の所 $ff しか観測できていないので詳細不明)
// e0,e1		SPC (BFFF、N=16 でミラー)
// e2-e7		繋がってない? 概ね $f4'00'c0'01 が読める。
//				FUSEROM などの繋がってないところと同じパターンが読める。
// f0,f1		たぶん Lance (BFFF、N=2)
// f2-fd		$ff
// fe			$fe00'0000-$fef7'ffff まではアドレスの残像
//				$fef8'0000-$feff'ffff までは $ff
// ff			たぶん CMMU 以外はアドレスの残像
//
// アドレスの残像(?) は
// - バイトアクセスならアドレスの下位 8bit、
// - ワードアクセスならアドレスの下位 16bit、
// - ロングワードアクセスならアドレス 32bit
// が見え、時々 $ffffffff が混ざるようだ ($00..$ff までのうち2個程度)。
// 一方、デバイスに接続されていない下位24ビットあるいは28ビットに
// $f400c001 が見えるエリアも多いが、$f400c001 は jmp r1 命令なので、おそらく
// PROM モニターがこのダンプを表示する際のプログラムコードがデータバスバッファ
// を通過したのが見えている可能性がある。
// いずれも再現は難しいので見なかったことにする。

/*static*/ const uint8
VM_LUNA88K::maptable[256] = {
	// RAM は Init() で動的に配置する (64MB なら 00..03 の4枠)。
	// RAM 未実装領域については要調査。少なくとも末尾に BUSERR がないと
	// OpenBSD カーネルが起動しない。
	// 先頭 64MB の空間 (00..03) だけは、PROM 1.20 ではメモリが無いときでも
	// バスエラーにならないことを期待しているので NONE にしておく。
	NONE,	NONE,	NONE,	NONE,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 00
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 08
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 10
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 18
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 20
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 28
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 30
	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	BUSERR,	// 38

	// LUNA-88K2 は 47 に(も?) NVRAM がいるようだ
	// 69,6b,6d にも何かのコントローラ?
	PROM,	PROM,	FUSE,	FUSE,	MK48,	MK48,	NONE,	NONE,	// 40
	PIO0,	PIO0,	NONE,	NONE,	PIO1,	PIO1,	NONE,	NONE,	// 48
	SIO,	SIO,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 50
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 58
	TAS,	TAS,	SYSCLK,	SYSCLK,	SYSCTL,	SYSCTL,	SYSCTL,	SYSCTL,	// 60
	SYSCTL,	SYSCTL,	SYSCTL,	SYSCTL,	SYSCTL,	SYSCTL,	SYSCTL,	SYSCTL,	// 68
	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	// 70
	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	SUBRAM,	// 78

	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 80
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 88
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 90
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// 98
	NONE,	MROM,	NONE,	MROM,	NONE,	MROM,	NONE,	MROM,	// a0
	NONE,	MROM,	NONE,	MROM,	NONE,	MROM,	NONE,	MROM,	// a8
	NONE,	LUNAFB,	NONE,	LUNAFB,	NONE,	LUNAFB,	NONE,	LUNAFB,	// b0
	NONE,	LUNAFB,	NONE,	LUNAFB,	NONE,	LUNAFB,	NONE,	LUNAFB,	// b8

	// d0 にボードチェックレジスタ?
	NONE,	BT454,	NONE,	BT454,	NONE,	BT454,	NONE,	BT454,	// c0
	NONE,	BT454,	NONE,	BT454,	NONE,	BT454,	NONE,	BT454,	// c8
	NONE,	CRTC2,	NONE,	CRTC2,	NONE,	CRTC2,	NONE,	CRTC2,	// d0
	NONE,	CRTC2,	NONE,	CRTC2,	NONE,	CRTC2,	NONE,	CRTC2,	// d8
	SCSI,	SCSI,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// e0
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// e8
	LANCE,	LANCE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	// f0
	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	NONE,	CMMU,	// f8
};

// コンストラクタ
VM_LUNA88K::VM_LUNA88K()
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 25);
	// RAM 初期値は MAX にしておく
	gConfig->SetDefault("ram-size", 64);
	// SPC の入力クロック (XXX LUNA-I と同じ?)
	gConfig->Add("!spc-tCLF", 163);
	// LUNA-II, LUNA-88K, LUNA-88K2 のクロック割り込みは 100Hz らしい
	gConfig->Add("!sysclk-period", 10_msec);

	// デバイス (順序に注意)
	// BusErrDevice, NopIODevice は親コンストラクタで作成済み。

	NEWDV(MPU, new MPU88xx0Device());
	NEWDV(Interrupt, new SysCtlrDevice());
	if (gMainApp.exec_file || gConfig->Find("prom-image").AsString().empty()) {
		NEWDV(PROM, new Luna88kPROMEmuDevice());
	} else {
		NEWDV(PROM, new PROMDevice());				// required by RAM
	}
	NEWDV(PROM0, new PROM0Device());
	NEWDV(MainRAM, new MainRAMDevice());			// required by PartialRAM
	NEWDV(PartialRAM, new PartialRAMDevice());
	NEWDV(FuseROM, new BusIO_BFFF<FuseROMDevice>());
	NEWDV(RTC, new BusIO_BFFF<MK48T02Device>());
	NEWDV(PIO0, new BusIO_BFFF<PIO0Device>());
	NEWDV(PIO1, new BusIO_BFFF<PIO1Device>());
	NEWDV(SIO, new BusIO_BFFF<SIODevice>());
	NEWDV(SysClk, new BusIO_BFFF<SysClkDevice>());
	NEWDV(TAS, new BusIO_BFFF<TASDevice>());
	NEWDV(SubRAM, new SubRAMDevice(128));
	NEWDV(BT454, new BT454Device());				// required by Lunafb
	NEWDV(PlaneVRAM, new LunafbDevice());
	NEWDV(CRTC2, new BusIO_BFFF<CRTC2Device>());
	NEWDV(SPC, new BusIO_BFFF<SPCDevice>());
	NEWDV(Ethernet, new BusIO_WF<LanceDevice>());
	NEWDV(CMMU, new CMMUDevice());

	NEWDV(NMI, new NMIDevice());
	NEWDV(LCD, new LCDDevice());
	NEWDV(Keyboard, new LunaKeyboard());
	NEWDV(Renderer, new LunaRenderer());

	// デバイスマップから devtable を作成
	MakeDevtable(maptable);
}

// デストラクタ
VM_LUNA88K::~VM_LUNA88K()
{
}

==> nono-0.6.3.tar.gz: nono-0.6.3/vm/vm_luna.cpp <==
//
// nono
// Copyright (C) 2020 nono project
// Licensed under nono-license.txt
//

//
// VM (LUNA)
//

#include "vm_luna.h"
#include "config.h"
#include "dipsw.h"
#include "lcd.h"
#include "lunakbd.h"
#include "mainapp.h"
#include "mainbus_luna.h"
#include "mpu64180.h"
#include "mpu680x0.h"
#include "mpu88xx0.h"
#include "msxdos.h"
#include "nmi.h"
#include "power.h"
#include "renderer.h"
#include "xpbus.h"

//
// LUNA 共通部
//

// コンストラクタ
VM_LUNA::VM_LUNA()
{
	// RTC epoch は NetBSD/luna68k, OpenBSD/luna88k をデフォルトにしておく。
	gConfig->SetDefault("rtc-epoch-year", 1970);

	// LUNA 共通だしここで作成しておく
	NEWDV(Dipsw, new LunaDipswDevice());
	NEWDV(Keyboard, new LunaKeyboard());
	NEWDV(LCD, new LCDDevice());
	NEWDV(MPU64180, new MPU64180Device());
	NEWDV(NMI, new NMIDevice());
	NEWDV(Renderer, new LunaRenderer());
	NEWDV(XPbus, new XPbusDevice());
}

// デストラクタ
VM_LUNA::~VM_LUNA()
{
}


//
// LUNA-I
//

// コンストラクタ
VM_LUNA1::VM_LUNA1()
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 20);
	// RAM 初期値は MAX にしておく
	gConfig->SetDefault("ram-size", 16);
	// SPC の入力クロック (おそらく 6.144MHz = 163ns が入ってる)
	gConfig->Add("!spc-tCLF", 163);
	// LUNA-I のクロック割り込みは 60Hz
	gConfig->Add("!sysclk-freq", 60);

	// デバイス
	NEWDV(MPU, new MPU680x0Device());
	NEWDV(Mainbus, new Luna1Mainbus());
}

// デストラクタ
VM_LUNA1::~VM_LUNA1()
{
}


//
// LUNA-88K
//

// コンストラクタ
VM_LUNA88K::VM_LUNA88K()
{
	// 機種固有パラメータ
	// MPU クロックの初期値は機種ごとに異なる。単位は MHz
	gConfig->SetDefault("mpu-clock", 25);
	// RAM 初期値は MAX にしておく
	gConfig->SetDefault("ram-size", 64);
	// SPC の入力クロック (XXX LUNA-I と同じ?)
	gConfig->Add("!spc-tCLF", 163);
	// LUNA-II, LUNA-88K, LUNA-88K2 のクロック割り込みは 100Hz らしい
	gConfig->Add("!sysclk-freq", 100);

	// デバイス (順序に注意)
	NEWDV(MPU, new MPU88xx0Device());
	NEWDV(Mainbus, new Luna88kMainbus());
}

// デストラクタ
VM_LUNA88K::~VM_LUNA88K()
{
}
